﻿using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using GTA;
using Utils;

#if USING_XBOX_PAD
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Input;
#endif

namespace NomeScripts
{
    public class SimulateKeyPress : Script
    {
#if USING_XBOX_PAD
        // Store pad buttons with actions
        X360PadVKEmulate gamePad;
#endif

        public SimulateKeyPress()
        {
            Interval = 100;

#if USING_XBOX_PAD
            gamePad = new X360PadVKEmulate(1);
#endif
            if (!LoadSettings())
            {
                Game.Console.Print("Loading INI data has failed");
            }

#if USING_XBOX_PAD
            if (gamePad.CurrentState.IsConnected)
            {
                // Set Tick event to monitor Controller Input
                this.Tick += new EventHandler(this.Input_Tick);
            }
#endif
        }

        private bool LoadSettings()
        {
            // Get the directory of the ini file
            int index = Filename.LastIndexOf("\\");
            String dir = Filename.Remove(index);            

#if USING_XBOX_PAD
            // Load XBOX settings
            if (!gamePad.LoadVKEmuFromFile(dir + "\\SimulateKeyPress.txt"))
            {
                return false;
            }
#endif
            return true;
        }

#if USING_XBOX_PAD
        // Controller Input
        private void Input_Tick(object sender, EventArgs e)
        {
            gamePad.UpdateVirtualKeys();

            ProcessKeys();
        }

        private void ProcessKeys()
        {
            // Create a copy of the pipeline
            ArrayList arr = new ArrayList(gamePad.keyPipeLine);

            // For each VirtualKey array in teh pipeline
            foreach (VirtualKey[] vKeyArr in gamePad.keyPipeLine)
            {
                // Send every key in the array as being pressed
                for (int i = 0; i < vKeyArr.Length; i++)
                {
                    if (vKeyArr[i].HoldKey) { // Hold all keys
                        Game.Console.Print(vKeyArr[i].KeyToEmulate.ToString());
                        HoldKey(vKeyArr[i]);
                    } else if (!vKeyArr[i].HoldKey) {// otherwise press each key individually
                        PressKey(vKeyArr[i], vKeyArr.Length);
                    }

                    // Remove the array of keys
                    if (i == vKeyArr.Length - 1 &&
                        vKeyArr[i].IsKeyIdle())
                    {
                        arr.Remove(vKeyArr);
                    }
                }
            }

            gamePad.keyPipeLine = arr;
        }

        private void PressKey(VirtualKey vKey, int arrLength)
        {
            if (arrLength > 1)
            {
                VKEmulate.SendKeyDown((int)vKey.KeyToEmulate);
                Wait(1);
            }

            VKEmulate.SendKeyPress((int)vKey.KeyToEmulate);

            if (arrLength > 1)
            {
                Wait(1);
                VKEmulate.SendKeyUp((int)vKey.KeyToEmulate);
                vKey.SetKeyState(VirtualKey.keyState.keyIdle);
            }
        }

        private void HoldKey(VirtualKey vKey)
        {
            if (vKey.IsKeyIdle())
            {
                vKey.SetKeyState(VirtualKey.keyState.KeyDown);
            }
            else if (vKey.IsKeyDown())
            {
                if (vKey.IsModifier)
                {
                    VKEmulate.SendKeyDown((int)vKey.Modifier);
                } else {
                    VKEmulate.SendKeyDown((int)vKey.KeyToEmulate);
                }

                vKey.SetKeyState(VirtualKey.keyState.KeyUp);
            }
            else if (vKey.IsKeyUp())
            {
                if (vKey.IsModifier)
                {
                    VKEmulate.SendKeyUp((int)vKey.Modifier);
                } else {
                    VKEmulate.SendKeyUp((int)vKey.KeyToEmulate);
                }

                vKey.SetKeyState(VirtualKey.keyState.keyIdle);
            }
        }
#endif
    }
}