﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace NomeScripts
{
    public class VirtualKey
    {
        public System.Windows.Forms.Keys KeyToEmulate { get { return keyToEmulate; } }
        System.Windows.Forms.Keys keyToEmulate;
        public bool IsModifier { get { return isModifier; } }
        bool isModifier;
        public System.Windows.Forms.Keys Modifier { get { return modifier; } }
        System.Windows.Forms.Keys modifier;
        /// <summary>
        /// True if this key must be held, false it it must be pressed
        /// </summary>
        public bool HoldKey { get { return holdKey; } }
        private bool holdKey;

        public enum keyState
        {
            keyIdle,
            KeyDown,
            KeyUp
        }

        private keyState state;

        public VirtualKey(System.Windows.Forms.Keys key, bool pressOrHold)
        {
            String[] mod = key.ToString().Split(',');
            if (mod.Length > 1)
            {
                System.Windows.Forms.KeysConverter kc = new System.Windows.Forms.KeysConverter();
                modifier = (System.Windows.Forms.Keys)kc.ConvertFromString(mod[1]);
                keyToEmulate = (System.Windows.Forms.Keys)kc.ConvertFromString(mod[0]);
                isModifier = true;
            }
            else
            {
                // Store keys to press
                keyToEmulate = key;
                isModifier = false;
            }

            holdKey = pressOrHold;
            state = keyState.keyIdle;
        }

        public void SetKeyState(keyState newState)
        {
            state = newState;
        }

        public bool IsKeyDown()
        {
            return (state == keyState.KeyDown);
        }

        public bool IsKeyUp()
        {
            return (state == keyState.KeyUp);
        }

        public bool IsKeyIdle()
        {
            return (state == keyState.keyIdle);
        }
    }

    public static class VKEmulate
    {
        [DllImport("user32.dll")]
        internal static extern uint MapVirtualKey(int uCode, int uMapType);
        [DllImport("user32.dll", SetLastError = true)]
        private static extern uint SendInput(uint numberOfInputs, INPUT[] inputs, int sizeOfInputStructure);

        const int INPUT_MOUSE = 0; // The event is a mouse event. Use the mi structure of the union.
        const int INPUT_KEYBOARD = 1; // The event is a keyboard event. Use the ki structure of the union.
        const int INPUT_HARDWARE = 2; // The event is a hardware event. Use the hi structure of the union.

        const int KEYEVENTF_EXTENDEDKEY = 0x0001;
        const int KEYEVENTF_KEYUP = 0x0002;
        const int KEYEVENTF_SCANCODE = 0x0008;
        const int KEYEVENTF_UNICODE = 0x0004;

        const int MAPVK_VK_TO_VSC = 0; // uCode is a virtual-key code and is translated into a scan code. If it is a virtual-key code that does not distinguish between left- and right-hand keys, the left-hand scan code is returned. If there is no translation, the function returns 0.
        const int MAPVK_VSC_TO_VK = 1; // uCode is a scan code and is translated into a virtual-key code that does not distinguish between left- and right-hand keys. If there is no translation, the function returns 0.
        const int MAPVK_VK_TO_CHAR = 2; // uCode is a virtual-key code and is translated into an unshifted character value in the low-order word of the return value. Dead keys (diacritics) are indicated by setting the top bit of the return value. If there is no translation, the function returns 0.
        const int MAPVK_VSC_TO_VK_EX = 3; // uCode is a scan code and is translated into a virtual-key code that distinguishes between left- and right-hand keys. If there is no translation, the function returns 0.

        public static void SendKeyPress(int keyCode)
        {
            uint scanCode = MapVirtualKey((int)keyCode, MAPVK_VK_TO_VSC);

            INPUT input = new INPUT
            {
                Type = INPUT_KEYBOARD
            };
            input.Data.Keyboard = new KEYBDINPUT()
            {
                Vk = (ushort)keyCode,
                Scan = (ushort)scanCode,
                Flags = 0,
                Time = 0,
                ExtraInfo = IntPtr.Zero,
            };

            INPUT input2 = new INPUT
            {
                Type = INPUT_KEYBOARD
            };
            input2.Data.Keyboard = new KEYBDINPUT()
            {
                Vk = (ushort)keyCode,
                Scan = (ushort)scanCode,
                Flags = KEYEVENTF_KEYUP,
                Time = 0,
                ExtraInfo = IntPtr.Zero
            };
            INPUT[] inputs = new INPUT[] { input, input2 };
            if (SendInput((uint)inputs.Length, inputs, Marshal.SizeOf(typeof(INPUT))) == 0)
                throw new Exception();
        }

        public static void SendKeyDown(int keyCode)
        {
            uint scanCode = MapVirtualKey((int)keyCode, MAPVK_VK_TO_VSC);

            INPUT input = new INPUT
            {
                Type = INPUT_KEYBOARD
            };
            input.Data.Keyboard = new KEYBDINPUT();
            input.Data.Keyboard.Vk = (ushort)keyCode;
            input.Data.Keyboard.Scan = (ushort)scanCode;
            input.Data.Keyboard.Flags = 0;
            input.Data.Keyboard.Time = 0;
            input.Data.Keyboard.ExtraInfo = IntPtr.Zero;
            INPUT[] inputs = new INPUT[] { input };

            if (SendInput((uint)inputs.Length, inputs, Marshal.SizeOf(typeof(INPUT))) == 0)
            {
                throw new Exception();
            }
        }

        public static void SendKeyUp(int keyCode)
        {
            uint scanCode = MapVirtualKey(keyCode, MAPVK_VK_TO_VSC);

            INPUT input = new INPUT
            {
                Type = INPUT_KEYBOARD
            };
            input.Data.Keyboard = new KEYBDINPUT();
            input.Data.Keyboard.Vk = (ushort)keyCode;
            input.Data.Keyboard.Scan = (ushort)scanCode;
            input.Data.Keyboard.Flags = KEYEVENTF_KEYUP;
            input.Data.Keyboard.Time = 0;
            input.Data.Keyboard.ExtraInfo = IntPtr.Zero;
            INPUT[] inputs = new INPUT[] { input };
            if (SendInput((uint)inputs.Length, inputs, Marshal.SizeOf(typeof(INPUT))) == 0)
                throw new Exception();

        }

        /// <summary>
        /// http://msdn.microsoft.com/en-us/library/windows/desktop/ms646270(v=vs.85).aspx
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        internal struct INPUT
        {
            public uint Type;
            public MOUSEKEYBDHARDWAREINPUT Data;
        }

        /// <summary>
        /// http://social.msdn.microsoft.com/Forums/en/csharplanguage/thread/f0e82d6e-4999-4d22-b3d3-32b25f61fb2a
        /// </summary>
        [StructLayout(LayoutKind.Explicit)]
        internal struct MOUSEKEYBDHARDWAREINPUT
        {
            [FieldOffset(0)]
            public HARDWAREINPUT Hardware;
            [FieldOffset(0)]
            public KEYBDINPUT Keyboard;
            [FieldOffset(0)]
            public MOUSEINPUT Mouse;
        }

        /// <summary>
        /// http://msdn.microsoft.com/en-us/library/windows/desktop/ms646310(v=vs.85).aspx
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        internal struct HARDWAREINPUT
        {
            public uint Msg;
            public ushort ParamL;
            public ushort ParamH;
        }

        /// <summary>
        /// http://msdn.microsoft.com/en-us/library/windows/desktop/ms646310(v=vs.85).aspx
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        internal struct KEYBDINPUT
        {
            public ushort Vk;
            public ushort Scan;
            public uint Flags;
            public uint Time;
            public IntPtr ExtraInfo;
        }

        /// <summary>
        /// http://social.msdn.microsoft.com/forums/en-US/netfxbcl/thread/2abc6be8-c593-4686-93d2-89785232dacd
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        internal struct MOUSEINPUT
        {
            public int X;
            public int Y;
            public uint MouseData;
            public uint Flags;
            public uint Time;
            public IntPtr ExtraInfo;
        }
    }
}
