﻿#if USING_XBOX_PAD

using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;
using System.Windows;
using System.Runtime.InteropServices;
using System.IO;
using System.Threading;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using GTA;

namespace NomeScripts
{
    public class X360PadVKEmulate
    {
        // Current gamepad state.
        public GamePadState CurrentState 
        { 
            get { return currentState; }
        }

        public Dictionary<Buttons[], VirtualKey[]> VKMapping
        {
            get { return vkMapping; }
        }

        public ArrayList keyPipeLine;
        Dictionary<Buttons[], VirtualKey[]> vkMapping;        
        ControllerParser controllerParser;
        GamePadState currentState;
        int connectedPort = -1;

        public X360PadVKEmulate(int portToConnect)
        {
            keyPipeLine = new ArrayList();
            controllerParser = new ControllerParser();
            vkMapping = new Dictionary<Buttons[], VirtualKey[]>();

            switch (portToConnect)
            {
                case 1 :
                    if (GamePad.GetState(PlayerIndex.One).IsConnected)
                    {
                        // Get state of the pad in Port one
                        currentState = GamePad.GetState(PlayerIndex.One);
                        connectedPort = portToConnect;
                    }
                    break;
                case 2 :
                    if (GamePad.GetState(PlayerIndex.Two).IsConnected)
                    {
                        // Get state of the pad in Port two
                        currentState = GamePad.GetState(PlayerIndex.Two);
                        connectedPort = portToConnect;
                    }
                    break;
                case 3 :
                    if (GamePad.GetState(PlayerIndex.Three).IsConnected)
                    {
                        // Get state of the pad in Port three
                        currentState = GamePad.GetState(PlayerIndex.Three);
                        connectedPort = portToConnect;
                    }
                    break;
                case 4 :
                    if (GamePad.GetState(PlayerIndex.Four).IsConnected)
                    {
                        // Get state of the pad in Port four
                        currentState = GamePad.GetState(PlayerIndex.Four);
                        connectedPort = portToConnect;
                    }
                    break;
                default:
                    Game.Console.Print("X360Pad : Port " + portToConnect.ToString() + " is not Connected");
                    portToConnect = -1;
                    break;
            }
        }

        public bool LoadVKEmuFromFile(String fileVKDir)
        {
            // Open file and parse out data
            StreamReader streamReader;
            try
            {
                streamReader = new StreamReader(fileVKDir);
            }
            catch (FileNotFoundException)
            {
                Game.Console.Print("SimulateKeyPress: File not found for loading keys");
                return false;
            }

            int lineNum = 0;
            while (!streamReader.EndOfStream)
            {
                lineNum++;
                String lineOfText = streamReader.ReadLine();
                lineOfText.Trim();

                // Check for file comments or lines beginning with white space or new line character
                if (lineOfText.Contains("#") || lineOfText[0] == ' ' ||  lineOfText[0] == '\n')
                    continue;

                String[] text = lineOfText.Split('=');
                
                // Get Xbox buttons
                Buttons[] xButArr = controllerParser.MapButtons(text[0]);

                for (int i = 0; i < xButArr.Length; ++i)
                {
                    if (!controllerParser.CheckButtonIsValid(xButArr[i]))
                    {
                        streamReader.Close();
                        vkMapping.Clear();
                        return false;
                    }
                }

                // Get Virtual Keys
                text[1] = text[1].Trim();
                char start = text[1].ElementAt(0);
                char end = text[1].ElementAt(text[1].Length-1);
                bool holdKeys = false;

                // Check if the virtual keys on this line are surrounded by {}, if they are then
                // this sequence of keys must be held at the same time
                if (start == '{' && end == '}')
                {
                    holdKeys = true;
                    text[1] = text[1].Replace('{', ' ');
                    text[1] = text[1].Replace('}', ' ');
                    text[1] = text[1].Trim();
                }

                // Split up each key
                String[] virtualKeys = text[1].Split(',');
                VirtualKey[] keys = new VirtualKey[virtualKeys.Length];

                for (int i = 0 ; i < virtualKeys.Length; i++)
                {
                    System.Windows.Forms.Keys k;
                    string key = ""; // To keep track of any issues

                    //start = virtualKeys[i].ElementAt(0);
                    //end = virtualKeys[i].ElementAt(virtualKeys[i].Length - 1);
                    //holdKeys = false;

                    //// Check if the virtual keys on this line are surrounded by {}, if they are then
                    //// this sequence of keys must be held at the same time
                    //if (start == '{' && end == '}')
                    //{
                    //    holdKeys = true;
                    //    virtualKeys[i] = virtualKeys[i].Replace('{', ' ');
                    //    virtualKeys[i] = virtualKeys[i].Replace('}', ' ');
                    //    virtualKeys[i] = virtualKeys[i].Trim();
                    //}

                    try
                    {
                        key = virtualKeys[i].Trim();
                        k = controllerParser.StringToKeyboardKey(key);
                    } catch (ArgumentException) {
                        Game.Console.Print("SimulateKeyPress: Check your Keyboard data on line " + lineNum.ToString() + " with " + key);
                        streamReader.Close();
                        vkMapping.Clear();
                        return false;
                    }

                    if (!controllerParser.CheckKeyIsValid(k))
                    {
                        streamReader.Close();
                        vkMapping.Clear();
                        return false;
                    }

                    keys[i] = new VirtualKey(k, holdKeys);
                }
                
                vkMapping.Add(xButArr, keys);
            }

            streamReader.Close();

            return true;
        }

        private void UpdatePadState()
        {
            switch (connectedPort)
            {
                case 1:
                    // Get state of the pad in Port one
                    currentState = GamePad.GetState(PlayerIndex.One);
                    break;
                case 2:
                    // Get state of the pad in Port two
                    currentState = GamePad.GetState(PlayerIndex.Two);
                    break;
                case 3:
                    // Get state of the pad in Port three
                    currentState = GamePad.GetState(PlayerIndex.Three);
                    break;
                case 4:
                    // Get state of the pad in Port four
                    currentState = GamePad.GetState(PlayerIndex.Four);
                    break;
                default:
                    Game.Console.Print("X360Pad: Cannot get state of controller");
                    break;
            }
        }

        public void UpdateVirtualKeys()
        {
            // Check all Pad Buttons
            // Check its still connected
            if (CurrentState.IsConnected)
            {
                // Get the current gamepad state.
                UpdatePadState();

                bool allButtonsArePressed;

                // Check each array of buttons in the keylist
                foreach (Buttons[] buttonArr in vkMapping.Keys)
                {
                    allButtonsArePressed = false;

                    foreach (Buttons button in buttonArr)
                    {
                        if (CurrentState.IsButtonDown(button))
                        {
                            allButtonsArePressed = true;
                        }
                        else
                        {
                            allButtonsArePressed = false;
                            break;
                        }
                    }

                    if (allButtonsArePressed)
                    {
                        // Pass the array of keys into the Key pipeline
                        keyPipeLine.Add(vkMapping[buttonArr]);
                    }
                }
            }
        }
    }
}
#endif